import { setDiagnostics } from "@codemirror/lint";
import { EditorState, Transaction, EditorSelection, SelectionRange, StateEffect, StateField } from "@codemirror/state";
import { EditorView } from "@codemirror/view";
import { memo } from "preact/compat";
import { useRef, useEffect, useState, useCallback } from "preact/hooks";
import { h } from "preact";

function $7b88c9dbbbf530897ebfb595ae51a7bd$var$_extends() {
  $7b88c9dbbbf530897ebfb595ae51a7bd$var$_extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return $7b88c9dbbbf530897ebfb595ae51a7bd$var$_extends.apply(this, arguments);
}

const $7b88c9dbbbf530897ebfb595ae51a7bd$var$setReadOnlyEffect = StateEffect.define();
const $7b88c9dbbbf530897ebfb595ae51a7bd$var$readOnlyState = StateField.define({
  create() {
    return true;
  },

  update(value, tr) {
    for (let effect of tr.effects) {
      if (effect.is($7b88c9dbbbf530897ebfb595ae51a7bd$var$setReadOnlyEffect)) {
        value = effect.value;
      }
    }

    return value;
  }

});
export function createState(value, extensions) {
  return EditorState.create({
    doc: value,
    extensions: [...extensions, $7b88c9dbbbf530897ebfb595ae51a7bd$var$readOnlyState, EditorState.changeFilter.of(({
      startState
    }) => !startState.field($7b88c9dbbbf530897ebfb595ae51a7bd$var$readOnlyState, false))]
  });
}
const $7b88c9dbbbf530897ebfb595ae51a7bd$var$emittedStates = new WeakSet();
export const Codemirror = memo(function Codemirror({
  state,
  onChange = null,
  readOnly = false,
  diagnostics = null,
  ...rest
}) {
  if (!state) {
    throw new Error("Missing state!");
  }

  const container = useRef(null);
  const view = useRef(null);
  const dispatchRefs = useRef({});
  useEffect(() => {
    dispatchRefs.current.onChange = onChange;
  }, [onChange]); // --- State ---

  useEffect(() => {
    // (subsequent render)
    if (!view.current) {
      return;
    }

    if ($7b88c9dbbbf530897ebfb595ae51a7bd$var$emittedStates.has(state)) {
      return;
    }

    view.current.setState(state);
  }, [state]);
  useEffect(() => {
    // (first render)
    let v = view.current = new EditorView({
      state,
      dispatch: t => {
        const {
          onChange
        } = dispatchRefs.current;
        v.update([t]);

        if (onChange) {
          $7b88c9dbbbf530897ebfb595ae51a7bd$var$emittedStates.add(v.state);
          onChange(v.state, t);
        }
      }
    });
    container.current.appendChild(v.dom);
    return () => v.destroy();
  }, []); // --- readOnly ---

  useEffect(() => {
    // if readOnly changes (first and subsequent render)
    view.current.dispatch({
      effects: $7b88c9dbbbf530897ebfb595ae51a7bd$var$setReadOnlyEffect.of(readOnly)
    });
  }, [readOnly]);
  useEffect(() => {
    // reapply readOnly if the state was changed externally
    if ($7b88c9dbbbf530897ebfb595ae51a7bd$var$emittedStates.has(state)) {
      return;
    }

    view.current.dispatch({
      effects: $7b88c9dbbbf530897ebfb595ae51a7bd$var$setReadOnlyEffect.of(readOnly)
    });
  }, [readOnly, state]); // --- Diagnostics ---

  useEffect(() => {
    // if diagnostics changes (first and subsequent render)
    view.current.dispatch(setDiagnostics(view.current.state, diagnostics && diagnostics.length > 0 ? diagnostics : []));
  }, [diagnostics]);
  useEffect(() => {
    // reapply diagnostics if the state was changed externally
    if ($7b88c9dbbbf530897ebfb595ae51a7bd$var$emittedStates.has(state)) {
      return;
    }

    view.current.dispatch(setDiagnostics(view.current.state, diagnostics && diagnostics.length > 0 ? diagnostics : []));
  }, [diagnostics, state]);
  return h("div", $7b88c9dbbbf530897ebfb595ae51a7bd$var$_extends({
    ref: container
  }, rest));
});
export function CodemirrorEditor({
  value,
  onChange: _onChange,
  readOnly,
  diagnostics,
  extensions
}) {
  const [state, setState] = useState(createState(value, extensions));
  useEffect(() => {
    if (value !== state.doc.toString()) {
      setState(createState(value, extensions));
    }
  }, [value]);
  useEffect(() => {
    setState(createState(value, extensions));
  }, [extensions]);
  const onChange = useCallback((newState, transaction) => {
    setState(newState);

    if (!transaction.changes.empty) {
      _onChange(newState.doc.toString());
    }
  }, []);
  return h(Codemirror, {
    state: state,
    onChange: onChange,
    readOnly: readOnly,
    diagnostics: diagnostics,
    class: "editor"
  });
}
//# sourceMappingURL=index.es.js.map
